@@ -223,7 +223,9 @@ class acceptor_t final
 			{
 				const auto ep = m_acceptor.local_endpoint();
 				m_logger.warn( [&]{
-					return fmt::format( "server already started on {}", ep );
+					return fmt::format(
+							RESTINIO_FMT_FORMAT_STRING( "server already started on {}" ),
+							fmtlib_tools::streamed( ep ) );
 				} );
 				return;
 			}
@@ -238,7 +240,9 @@ class acceptor_t final
 			try
 			{
 				m_logger.trace( [&]{
-					return fmt::format( "starting server on {}", ep );
+					return fmt::format(
+							RESTINIO_FMT_FORMAT_STRING( "starting server on {}" ),
+							fmtlib_tools::streamed( ep ) );
 				} );
 
 				m_acceptor.open( ep.protocol() );
@@ -265,14 +269,17 @@ class acceptor_t final
 				for( std::size_t i = 0; i< this->concurrent_accept_sockets_count(); ++i )
 				{
 					m_logger.info( [&]{
-						return fmt::format( "init accept #{}", i );
+						return fmt::format(
+								RESTINIO_FMT_FORMAT_STRING( "init accept #{}" ), i );
 					} );
 
 					accept_next( i );
 				}
 
 				m_logger.info( [&]{
-					return fmt::format( "server started on {}", ep );
+					return fmt::format(
+							RESTINIO_FMT_FORMAT_STRING( "server started on {}" ),
+							fmtlib_tools::streamed( ep ) );
 				} );
 			}
 			catch( const std::exception & ex )
@@ -282,9 +289,11 @@ class acceptor_t final
 					m_acceptor.close();
 
 				m_logger.error( [&]() -> auto {
-					return fmt::format( "failed to start server on {}: {}",
-						ep,
-						ex.what() );
+					return fmt::format(
+							RESTINIO_FMT_FORMAT_STRING(
+									"failed to start server on {}: {}" ),
+							fmtlib_tools::streamed( ep ),
+							ex.what() );
 				} );
 
 				throw;
@@ -301,9 +310,12 @@ class acceptor_t final
 			}
 			else
 			{
-				m_logger.trace( [&]{
-					return fmt::format( "server already closed" );
-				} );
+				// v.0.7.0: suppress exceptions from logging.
+				restinio::utils::log_trace_noexcept( m_logger,
+					[&]{
+						return fmt::format(
+								RESTINIO_FMT_FORMAT_STRING( "server already closed" ) );
+					} );
 			}
 		}
 
@@ -409,7 +421,8 @@ class acceptor_t final
 				restinio::utils::log_error_noexcept( m_logger,
 					[&]{
 						return fmt::format(
-							"failed to accept connection on socket #{}: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"failed to accept connection on socket #{}: {}" ),
 							i,
 							ec.message() );
 					} );
@@ -439,8 +452,10 @@ class acceptor_t final
 
 			m_logger.trace( [&]{
 				return fmt::format(
-						"accept connection from {} on socket #{}",
-						remote_endpoint, i );
+						RESTINIO_FMT_FORMAT_STRING(
+							"accept connection from {} on socket #{}" ),
+						fmtlib_tools::streamed( remote_endpoint ),
+						i );
 			} );
 
 			// Since v.0.5.1 the incoming connection must be
@@ -454,9 +469,11 @@ class acceptor_t final
 				// New connection can be used. It is disabled by IP-blocker.
 				m_logger.warn( [&]{
 					return fmt::format(
-							"accepted connection from {} on socket #{} denied by"
-							" IP-blocker",
-							remote_endpoint, i );
+							RESTINIO_FMT_FORMAT_STRING(
+								"accepted connection from {} on socket #{} denied by"
+								" IP-blocker" ),
+							fmtlib_tools::streamed( remote_endpoint ),
+							i );
 				} );
 				// incoming_socket will be closed automatically.
 			break;
@@ -526,14 +543,20 @@ class acceptor_t final
 			// for m_acceptor.
 			restinio::utils::log_trace_noexcept( m_logger,
 				[&]{
-					return fmt::format( "closing server on {}", ep );
+					return fmt::format(
+							RESTINIO_FMT_FORMAT_STRING( "closing server on {}" ),
+							fmtlib_tools::streamed( ep ) );
 				} );
 
 			m_acceptor.close();
 
-			m_logger.info( [&]{
-				return fmt::format( "server closed on {}", ep );
-			} );
+			// v.0.7.0: Suppress exceptions from this logging too.
+			restinio::utils::log_info_noexcept( m_logger,
+				[&]{
+					return fmt::format(
+							RESTINIO_FMT_FORMAT_STRING( "server closed on {}" ),
+							fmtlib_tools::streamed( ep ) );
+				} );
 		}
 
 		//! Server endpoint.
@@ -582,14 +605,14 @@ class acceptor_t final
 		 *
 		 * @since v.0.6.11
 		 */
-		RESTINIO_NODISCARD
-		static optional_t< asio_ns::ip::address >
+		[[nodiscard]]
+		static std::optional< asio_ns::ip::address >
 		try_extract_actual_address_from_variant(
 			const restinio::details::address_variant_t & from )
 		{
-			optional_t< asio_ns::ip::address > result;
+			std::optional< asio_ns::ip::address > result;
 
-			if( auto * str_v = get_if<std::string>( &from ) )
+			if( auto * str_v = std::get_if<std::string>( &from ) )
 			{
 				auto str_addr = *str_v;
 				if( str_addr == "localhost" )
@@ -599,7 +622,7 @@ class acceptor_t final
 
 				result = asio_ns::ip::address::from_string( str_addr );
 			}
-			else if( auto * addr_v = get_if<asio_ns::ip::address>( &from ) )
+			else if( auto * addr_v = std::get_if<asio_ns::ip::address>( &from ) )
 			{
 				result = *addr_v;
 			}
