@@ -10,13 +10,14 @@
 
 #include <restinio/asio_include.hpp>
 
-#include <http_parser.h>
+#include <llhttp.h>
 
 #include <restinio/impl/include_fmtlib.hpp>
 
 #include <restinio/exception.hpp>
 #include <restinio/http_headers.hpp>
 #include <restinio/request_handler.hpp>
+#include <restinio/connection_count_limiter.hpp>
 #include <restinio/impl/connection_base.hpp>
 #include <restinio/impl/header_helpers.hpp>
 #include <restinio/impl/response_coordinator.hpp>
@@ -55,7 +56,7 @@ struct http_parser_ctx_t
 	//! Parser context temp values and flags.
 	//! \{
 	std::string m_current_field_name;
-	bool m_last_was_value{ true };
+	std::size_t m_last_value_total_size{ 0u };
 
 	/*!
 	 * @since v.0.6.9
@@ -66,11 +67,53 @@ struct http_parser_ctx_t
 	 * @since v.0.6.9
 	 */
 	chunked_input_info_block_t m_chunked_info_block;
+
+	/*!
+	 * @brief Chunk extnsion's params if any.
+	 *
+	 * @since v.0.7.0
+	 */
+	chunk_ext_params_unique_ptr_t m_chunk_ext_params;
+
+	/*!
+	 * @brief How many bytes were parsed for current request.
+	 *
+	 * @since v.0.7.0
+	 */
+	std::size_t m_bytes_parsed;
+
 	//! \}
 
 	//! Flag: is http message parsed completely.
 	bool m_message_complete{ false };
 
+	/*!
+	 * @brief Total number of parsed HTTP-fields.
+	 *
+	 * This number includes the number of leading HTTP-fields and the number
+	 * of trailing HTTP-fields (in the case of chunked encoding).
+	 *
+	 * @since v.0.6.12
+	 */
+	std::size_t m_total_field_count{ 0u };
+
+	/*!
+	 * @brief Limits for the incoming message.
+	 *
+	 * @since v.0.6.12
+	 */
+	const incoming_http_msg_limits_t m_limits;
+
+	/*!
+	 * @brief The main constructor.
+	 *
+	 * @since v.0.6.12
+	 */
+	http_parser_ctx_t(
+		incoming_http_msg_limits_t limits )
+		:	m_limits{ limits }
+	{}
+
 	//! Prepare context to handle new request.
 	void
 	reset()
@@ -78,9 +121,11 @@ struct http_parser_ctx_t
 		m_header = http_request_header_t{};
 		m_body.clear();
 		m_current_field_name.clear();
-		m_last_was_value = true;
+		m_last_value_total_size = 0u;
 		m_leading_headers_completed = false;
+		m_bytes_parsed = 0;
 		m_message_complete = false;
+		m_total_field_count = 0u;
 	}
 
 	//! Creates an instance of chunked_input_info if there is an info
@@ -88,7 +133,7 @@ struct http_parser_ctx_t
 	/*!
 	 * @since v.0.6.9
 	 */
-	RESTINIO_NODISCARD
+	[[nodiscard]]
 	chunked_input_info_unique_ptr_t
 	make_chunked_input_info_if_necessary()
 	{
@@ -117,52 +162,82 @@ struct http_parser_ctx_t
 	Is used to initialize const value in connection_settings_t ctor.
 */
 template< typename Http_Methods >
-inline http_parser_settings
+inline llhttp_settings_t
 create_parser_settings() noexcept
 {
-	http_parser_settings parser_settings;
-	http_parser_settings_init( &parser_settings );
+	llhttp_settings_t parser_settings;
+	llhttp_settings_init( &parser_settings );
 
 	parser_settings.on_url =
-		[]( http_parser * parser, const char * at, size_t length ) -> int {
+		[]( llhttp_t * parser, const char * at, size_t length ) -> int {
 			return restinio_url_cb( parser, at, length );
 		};
 
 	parser_settings.on_header_field =
-		[]( http_parser * parser, const char * at, size_t length ) -> int {
+		[]( llhttp_t * parser, const char * at, size_t length ) -> int {
 			return restinio_header_field_cb( parser, at, length );
 		};
 
+	parser_settings.on_header_field_complete =
+		[]( llhttp_t * parser ) -> int {
+			return restinio_header_field_complete_cb( parser );
+		};
+
 	parser_settings.on_header_value =
-			[]( http_parser * parser, const char * at, size_t length ) -> int {
+			[]( llhttp_t * parser, const char * at, size_t length ) -> int {
 			return restinio_header_value_cb( parser, at, length );
 		};
 
+	parser_settings.on_header_value_complete =
+		[]( llhttp_t * parser ) -> int {
+			return restinio_header_value_complete_cb( parser );
+		};
+
 	parser_settings.on_headers_complete =
-		[]( http_parser * parser ) -> int {
+		[]( llhttp_t * parser ) -> int {
 			return restinio_headers_complete_cb( parser );
 		};
 
 	parser_settings.on_body =
-		[]( http_parser * parser, const char * at, size_t length ) -> int {
+		[]( llhttp_t * parser, const char * at, size_t length ) -> int {
 			return restinio_body_cb( parser, at, length );
 		};
 
 	parser_settings.on_chunk_header =
-		[]( http_parser * parser ) -> int {
+		[]( llhttp_t * parser ) -> int {
 			return restinio_chunk_header_cb( parser );
 		};
 
 	parser_settings.on_chunk_complete =
-		[]( http_parser * parser ) -> int {
+		[]( llhttp_t * parser ) -> int {
 			return restinio_chunk_complete_cb( parser );
 		};
 
 	parser_settings.on_message_complete =
-		[]( http_parser * parser ) -> int {
+		[]( llhttp_t * parser ) -> int {
 			return restinio_message_complete_cb< Http_Methods >( parser );
 		};
 
+	parser_settings.on_chunk_extension_name =
+		[]( llhttp_t * parser, const char * at, size_t length  ) -> int {
+			return restinio_chunk_extension_name_cb( parser, at, length );
+		};
+
+	parser_settings.on_chunk_extension_value =
+		[]( llhttp_t * parser, const char * at, size_t length  ) -> int {
+			return restinio_chunk_extension_value_cb( parser, at, length );
+		};
+
+	parser_settings.on_chunk_extension_name_complete =
+		[]( llhttp_t * parser ) -> int {
+			return restinio_chunk_extension_name_complete_cb( parser );
+		};
+
+	parser_settings.on_chunk_extension_value_complete =
+		[]( llhttp_t * parser ) -> int {
+			return restinio_chunk_extension_value_complete_cb( parser );
+		};
+
 	return parser_settings;
 }
 
@@ -193,13 +268,20 @@ enum class connection_upgrade_stage_t : std::uint8_t
 //! Data associated with connection read routine.
 struct connection_input_t
 {
-	connection_input_t( std::size_t buffer_size )
-		:	m_buf{ buffer_size }
-	{}
+	connection_input_t(
+		std::size_t buffer_size,
+		incoming_http_msg_limits_t limits,
+		const llhttp_settings_t* settings )
+		:	m_parser_ctx{ limits }
+		,	m_buf{ buffer_size }
+	{
+		llhttp_init( &m_parser, llhttp_type_t::HTTP_REQUEST, settings );
+		m_parser.data = &m_parser_ctx;
+	}
 
 	//! HTTP-parser.
 	//! \{
-	http_parser m_parser;
+	llhttp_t m_parser;
 	http_parser_ctx_t m_parser_ctx;
 	//! \}
 
@@ -218,11 +300,10 @@ struct connection_input_t
 	reset_parser()
 	{
 		// Reinit parser.
-		http_parser_init( &m_parser, HTTP_REQUEST);
+		llhttp_reset( &m_parser);
 
 		// Reset context and attach it to parser.
 		m_parser_ctx.reset();
-		m_parser.data = &m_parser_ctx;
 	}
 };
 
@@ -275,10 +356,13 @@ class connection_t final
 	public:
 		using timer_manager_t = typename Traits::timer_manager_t;
 		using timer_guard_t = typename timer_manager_t::timer_guard_t;
-		using request_handler_t = typename Traits::request_handler_t;
+		using request_handler_t = request_handler_type_from_traits_t< Traits >;
+		using generic_request_t = generic_request_type_from_traits_t< Traits >;
 		using logger_t = typename Traits::logger_t;
 		using strand_t = typename Traits::strand_t;
 		using stream_socket_t = typename Traits::stream_socket_t;
+		using lifetime_monitor_t =
+				typename connection_count_limit_types<Traits>::lifetime_monitor_t;
 
 		connection_t(
 			//! Connection id.
@@ -288,24 +372,32 @@ class connection_t final
 			//! Settings that are common for connections.
 			connection_settings_handle_t< Traits > settings,
 			//! Remote endpoint for that connection.
-			endpoint_t remote_endpoint )
+			endpoint_t remote_endpoint,
+			//! Lifetime monitor to be used for handling connection count.
+			lifetime_monitor_t lifetime_monitor )
 			:	connection_base_t{ conn_id }
 			,	executor_wrapper_base_t{ socket.get_executor() }
 			,	m_socket{ std::move( socket ) }
 			,	m_settings{ std::move( settings ) }
 			,	m_remote_endpoint{ std::move( remote_endpoint ) }
-			,	m_input{ m_settings->m_buffer_size }
+			,	m_input{
+					m_settings->m_buffer_size,
+					m_settings->m_incoming_http_msg_limits,
+					&m_settings->m_parser_settings
+				}
 			,	m_response_coordinator{ m_settings->m_max_pipelined_requests }
 			,	m_timer_guard{ m_settings->create_timer_guard() }
 			,	m_request_handler{ *( m_settings->m_request_handler ) }
 			,	m_logger{ *( m_settings->m_logger ) }
+			,	m_lifetime_monitor{ std::move(lifetime_monitor) }
 		{
 			// Notify of a new connection instance.
 			m_logger.trace( [&]{
 					return fmt::format(
-						"[connection:{}] start connection with {}",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] start connection with {}" ),
 						connection_id(),
-						m_remote_endpoint );
+						fmtlib_tools::streamed( m_remote_endpoint ) );
 			} );
 		}
 
@@ -320,7 +412,8 @@ class connection_t final
 			restinio::utils::log_trace_noexcept( m_logger,
 				[&]{
 					return fmt::format(
-						"[connection:{}] destructor called",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] destructor called" ),
 						connection_id() );
 				} );
 		}
@@ -354,7 +447,8 @@ class connection_t final
 				[ & ]( const asio_ns::error_code & ec ){
 					trigger_error_and_close( [&]{
 						return fmt::format(
-								"[connection:{}] prepare connection error: {}",
+								RESTINIO_FMT_FORMAT_STRING(
+									"[connection:{}] prepare connection error: {}" ),
 								connection_id(),
 								ec.message() );
 					} );
@@ -367,7 +461,8 @@ class connection_t final
 		{
 			m_logger.trace( [&]{
 				 return fmt::format(
-						"[connection:{}] start waiting for request",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] start waiting for request" ),
 						connection_id() );
 			} );
 
@@ -402,13 +497,16 @@ class connection_t final
 
 			upgrade_internals_t(
 				connection_settings_handle_t< Traits > settings,
-				stream_socket_t socket )
-				:	m_settings{ std::move( settings ) }
+				stream_socket_t socket,
+				lifetime_monitor_t lifetime_monitor )
+				:	m_settings{ std::move(settings) }
 				,	m_socket{ std::move( socket ) }
+				,	m_lifetime_monitor{ std::move(lifetime_monitor) }
 			{}
 
 			connection_settings_handle_t< Traits > m_settings;
 			stream_socket_t m_socket;
+			lifetime_monitor_t m_lifetime_monitor;
 		};
 
 		//! Move socket out of connection.
@@ -417,7 +515,9 @@ class connection_t final
 		{
 			return upgrade_internals_t{
 				m_settings,
-				std::move( m_socket ) };
+				std::move(m_socket),
+				std::move(m_lifetime_monitor)
+			};
 		}
 
 	private:
@@ -429,7 +529,8 @@ class connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-							"[connection:{}] continue reading request",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] continue reading request" ),
 							connection_id() );
 				} );
 
@@ -451,7 +552,8 @@ class connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-							"[connection:{}] skip read operation: already running",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] skip read operation: already running" ),
 							connection_id() );
 				} );
 			}
@@ -470,7 +572,8 @@ class connection_t final
 				{
 					m_logger.trace( [&]{
 						return fmt::format(
-								"[connection:{}] received {} bytes",
+								RESTINIO_FMT_FORMAT_STRING(
+									"[connection:{}] received {} bytes" ),
 								this->connection_id(),
 								length );
 					} );
@@ -483,8 +586,9 @@ class connection_t final
 				{
 					trigger_error_and_close( [&] {
 							return fmt::format(
-									"[connection:{}] unexpected exception during the "
-									"handling of incoming data: {}",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[connection:{}] unexpected exception during the "
+										"handling of incoming data: {}" ),
 									connection_id(),
 									x.what() );
 						} );
@@ -496,14 +600,15 @@ class connection_t final
 				// then close connection.
 				if( !error_is_operation_aborted( ec ) )
 				{
-					if ( !error_is_eof( ec ) || 0 != m_input.m_parser.nread )
+					if ( !error_is_eof( ec ) || 0 != m_input.m_parser_ctx.m_bytes_parsed )
 						trigger_error_and_close( [&]{
 							return fmt::format(
-									"[connection:{}] read socket error: {}; "
-									"parsed bytes: {}",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[connection:{}] read socket error: {}; "
+										"parsed bytes: {}" ),
 									connection_id(),
 									ec.message(),
-									m_input.m_parser.nread );
+									m_input.m_parser_ctx.m_bytes_parsed );
 						} );
 					else
 					{
@@ -514,8 +619,9 @@ class connection_t final
 						restinio::utils::log_trace_noexcept( m_logger,
 							[&]{
 								return fmt::format(
-										"[connection:{}] EOF and no request, "
-										"close connection",
+										RESTINIO_FMT_FORMAT_STRING(
+											"[connection:{}] EOF and no request, "
+											"close connection" ),
 										connection_id() );
 							} );
 
@@ -530,14 +636,36 @@ class connection_t final
 		void
 		consume_data( const char * data, std::size_t length )
 		{
-			auto & parser = m_input.m_parser;
+			auto * parser = &m_input.m_parser;
 
-			const auto nparsed =
-				http_parser_execute(
-					&parser,
-					&( m_settings->m_parser_settings ),
-					data,
-					length );
+			const auto parse_err =
+				llhttp_execute( parser, data, length );
+
+			const auto nparsed = [&]{
+				if( !parser->error_pos )
+					return length;
+				return static_cast< std::size_t >( parser->error_pos - data );
+			}();
+
+			if( nparsed > length )
+			{
+				// Parser is in the unreliable state,
+				// so we done with this connection.
+				trigger_error_and_close( [&]{
+					return fmt::format(
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] unexpected parser behavior: "
+								"llhttp_execute() reports parsed bytes number ({}) "
+								"is greater than the size of a buffer ({})"
+								"that was fed to the parser" ),
+							connection_id(),
+							nparsed,
+							length );
+				} );
+				return;
+			}
+
+			m_input.m_parser_ctx.m_bytes_parsed += nparsed;
 
 			// If entire http-message was obtained,
 			// parser is stopped and the might be a part of consecutive request
@@ -546,19 +674,28 @@ class connection_t final
 			// data left in buffer.
 			m_input.m_buf.consumed_bytes( nparsed );
 
-			if( HPE_OK != parser.http_errno &&
-				HPE_PAUSED != parser.http_errno )
+			if( HPE_OK != parse_err &&
+				HPE_PAUSED != parse_err &&
+				HPE_PAUSED_UPGRADE != parse_err )
 			{
-				// PARSE ERROR:
-				auto err = HTTP_PARSER_ERRNO( &parser );
+				// Parser error must be the one defined by llhttp_errno.
+				// It is possible to get a random error value
+				// (e.g. providing parser-callbacks that return
+				// unconventional errors), here we put an assert
+				// to quickly highlight the problems.
+				// As all the parser-callbacks are implemented by
+				// restinio the assert bellow MUST pass.
+				assert( llhttp_errno::HPE_OK <= parse_err &&
+						llhttp_errno::HPE_CB_RESET >= parse_err );
 
 				// TODO: handle case when there are some request in process.
 				trigger_error_and_close( [&]{
 					return fmt::format(
-							"[connection:{}] parser error {}: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] parser error {}: {}" ),
 							connection_id(),
-							http_errno_name( err ),
-							http_errno_description( err ) );
+							llhttp_errno_name( parse_err ),
+							llhttp_get_error_reason( parser ) );
 				} );
 
 				// nothing to do.
@@ -601,11 +738,12 @@ class connection_t final
 
 					m_logger.trace( [&]{
 						return fmt::format(
-								"[connection:{}] request received (#{}): {} {}",
+								RESTINIO_FMT_FORMAT_STRING(
+									"[connection:{}] request received (#{}): {} {}" ),
 								connection_id(),
 								request_id,
-								http_method_str(
-									static_cast<http_method>( parser.method ) ),
+								llhttp_method_name(
+									static_cast<llhttp_method>( parser.method ) ),
 								parser_ctx.m_header.request_target() );
 					} );
 
@@ -615,31 +753,40 @@ class connection_t final
 					// so it is possible to omit this timer scheduling.
 					guard_request_handling_operation();
 
-					if( request_rejected() ==
+					const auto handling_result =
 						m_request_handler(
-							std::make_shared< request_t >(
+							std::make_shared< generic_request_t >(
 								request_id,
 								std::move( parser_ctx.m_header ),
 								std::move( parser_ctx.m_body ),
 								parser_ctx.make_chunked_input_info_if_necessary(),
 								shared_from_concrete< connection_base_t >(),
-								m_remote_endpoint ) ) )
+								m_remote_endpoint,
+								m_settings->extra_data_factory() ) );
+
+					switch( handling_result )
 					{
-						// If handler refused request, say not implemented.
-						write_response_parts_impl(
-							request_id,
-							response_output_flags_t{
-								response_parts_attr_t::final_parts,
-								response_connection_attr_t::connection_close },
-							write_group_t{ create_not_implemented_resp() } );
-					}
-					else if( m_response_coordinator.is_able_to_get_more_messages() )
-					{
-						// Request was accepted,
-						// didn't create immediate response that closes connection after,
-						// and it is possible to receive more requests
-						// then start consuming yet another request.
-						wait_for_http_message();
+						case request_handling_status_t::not_handled:
+						case request_handling_status_t::rejected:
+							// If handler refused request, say not implemented.
+							write_response_parts_impl(
+								request_id,
+								response_output_flags_t{
+									response_parts_attr_t::final_parts,
+									response_connection_attr_t::connection_close },
+								write_group_t{ create_not_implemented_resp() } );
+							break;
+
+						case request_handling_status_t::accepted:
+							if( m_response_coordinator.is_able_to_get_more_messages() )
+							{
+								// Request was accepted,
+								// didn't create immediate response that closes connection after,
+								// and it is possible to receive more requests
+								// then start consuming yet another request.
+								wait_for_http_message();
+							}
+							break;
 					}
 				}
 				else
@@ -648,11 +795,12 @@ class connection_t final
 						const std::string default_value{};
 
 						return fmt::format(
-								"[connection:{}] upgrade request received: {} {}; "
-								"Upgrade: '{}';",
+								RESTINIO_FMT_FORMAT_STRING(
+									"[connection:{}] upgrade request received: {} {}; "
+									"Upgrade: '{}';" ),
 								connection_id(),
-								http_method_str(
-									static_cast<http_method>( parser.method ) ),
+								llhttp_method_name(
+									static_cast<llhttp_method>( parser.method ) ),
 								parser_ctx.m_header.request_target(),
 								parser_ctx.m_header.get_field_or(
 									http_field::upgrade, default_value ) );
@@ -671,8 +819,11 @@ class connection_t final
 						// There are pipelined request
 						m_logger.trace( [&]{
 							return fmt::format(
-									"[connection:{}] upgrade request happened to be a pipelined one, "
-									"and will be handled after previous requests are handled",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[connection:{}] upgrade request happened to "
+										"be a pipelined one, "
+										"and will be handled after previous requests "
+										"are handled" ),
 									connection_id() );
 						} );
 					}
@@ -685,7 +836,8 @@ class connection_t final
 			{
 				trigger_error_and_close( [&]{
 					return fmt::format(
-							"[connection:{}] error while handling request: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] error while handling request: {}" ),
 							this->connection_id(),
 							ex.what() );
 				} );
@@ -710,11 +862,12 @@ class connection_t final
 
 			m_logger.info( [&]{
 				return fmt::format(
-						"[connection:{}] handle upgrade request (#{}): {} {}",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] handle upgrade request (#{}): {} {}" ),
 						connection_id(),
 						request_id,
-						http_method_str(
-							static_cast<http_method>( parser.method ) ),
+						llhttp_method_name(
+							static_cast<llhttp_method>( parser.method ) ),
 						parser_ctx.m_header.request_target() );
 			} );
 
@@ -726,41 +879,50 @@ class connection_t final
 			m_input.m_connection_upgrade_stage =
 				connection_upgrade_stage_t::wait_for_upgrade_handling_result_or_nothing;
 
-			if( request_rejected() ==
-				m_request_handler(
-					std::make_shared< request_t >(
-						request_id,
-						std::move( parser_ctx.m_header ),
-						std::move( parser_ctx.m_body ),
-						parser_ctx.make_chunked_input_info_if_necessary(),
-						shared_from_concrete< connection_base_t >(),
-						m_remote_endpoint) ) )
+			const auto handling_result = m_request_handler(
+				std::make_shared< generic_request_t >(
+					request_id,
+					std::move( parser_ctx.m_header ),
+					std::move( parser_ctx.m_body ),
+					parser_ctx.make_chunked_input_info_if_necessary(),
+					shared_from_concrete< connection_base_t >(),
+					m_remote_endpoint,
+					m_settings->extra_data_factory() ) );
+			switch( handling_result )
 			{
-				if( m_socket.is_open() )
-				{
-					// Request is rejected, so our socket
-					// must not be moved out to websocket connection.
+				case request_handling_status_t::not_handled:
+				case request_handling_status_t::rejected:
+					if( m_socket.is_open() )
+					{
+						// Request is rejected, so our socket
+						// must not be moved out to websocket connection.
 
-					// If handler refused request, say not implemented.
-					write_response_parts_impl(
-						request_id,
-						response_output_flags_t{
-							response_parts_attr_t::final_parts,
-							response_connection_attr_t::connection_close },
-						write_group_t{ create_not_implemented_resp() } );
-				}
-				else
-				{
-					// Request is rejected, but the socket
-					// was moved out to somewhere else???
+						// If handler refused request, say not implemented.
+						write_response_parts_impl(
+							request_id,
+							response_output_flags_t{
+								response_parts_attr_t::final_parts,
+								response_connection_attr_t::connection_close },
+							write_group_t{ create_not_implemented_resp() } );
+					}
+					else
+					{
+						// Request is rejected, but the socket
+						// was moved out to somewhere else???
 
-					m_logger.error( [&]{
-						return fmt::format(
-								"[connection:{}] upgrade request handler rejects "
-								"request, but socket was moved out from connection",
-								connection_id() );
-					} );
-				}
+						m_logger.error( [&]{
+							return fmt::format(
+									RESTINIO_FMT_FORMAT_STRING(
+										"[connection:{}] upgrade request handler rejects "
+										"request, but socket was moved out from connection" ),
+									connection_id() );
+						} );
+					}
+					break;
+
+				case request_handling_status_t::accepted:
+					/* nothing to do */
+					break;
 			}
 
 			// Else 2 cases:
@@ -808,7 +970,8 @@ class connection_t final
 						{
 							trigger_error_and_close( [&]{
 								return fmt::format(
-									"[connection:{}] unable to handle response: {}",
+									RESTINIO_FMT_FORMAT_STRING(
+										"[connection:{}] unable to handle response: {}" ),
 									connection_id(),
 									ex.what() );
 							} );
@@ -837,7 +1000,8 @@ class connection_t final
 				{
 					m_logger.error( [&]{
 						return fmt::format(
-							"[connection:{}] notificator error: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] notificator error: {}" ),
 							connection_id(),
 							ex.what() );
 					} );
@@ -867,11 +1031,12 @@ class connection_t final
 				{
 					m_logger.trace( [&]{
 						return fmt::format(
-							"[connection:{}] append response (#{}), "
-							"flags: {}, write group size: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] append response (#{}), "
+								"flags: {}, write group size: {}" ),
 							connection_id(),
 							request_id,
-							response_output_flags,
+							fmtlib_tools::streamed( response_output_flags ),
 							wg.items_count() );
 					} );
 
@@ -886,9 +1051,10 @@ class connection_t final
 				{
 					m_logger.warn( [&]{
 						return fmt::format(
-								"[connection:{}] receive response parts for "
-								"request (#{}), but response with connection-close "
-								"attribute happened before",
+								RESTINIO_FMT_FORMAT_STRING(
+									"[connection:{}] receive response parts for "
+									"request (#{}), but response with connection-close "
+									"attribute happened before" ),
 								connection_id(),
 								request_id );
 					} );
@@ -899,8 +1065,9 @@ class connection_t final
 			{
 				m_logger.warn( [&]{
 					return fmt::format(
-							"[connection:{}] try to write response, "
-							"while socket is closed",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] try to write response, "
+								"while socket is closed" ),
 							connection_id() );
 				} );
 				invoke_after_write_cb_with_error();
@@ -937,8 +1104,9 @@ class connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-						"[connection:{}] start next write group for response (#{}), "
-						"size: {}",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] start next write group for response (#{}), "
+							"size: {}" ),
 						this->connection_id(),
 						next_write_group->second,
 						next_write_group->first.items_count() );
@@ -962,18 +1130,17 @@ class connection_t final
 
 					m_logger.trace( [&]{
 						// Get status line:
-						const string_view_t
-							status_line{
+						const string_view_t status_line{
 								asio_ns::buffer_cast< const char * >(
 									next_write_group->first.items().front().buf() ),
 								next_write_group->first.status_line_size() };
 
-						return
-							fmt::format(
-								"[connection:{}] start response (#{}): {}",
+						return fmt::format(
+								RESTINIO_FMT_FORMAT_STRING(
+									"[connection:{}] start response (#{}): {}" ),
 								this->connection_id(),
 								next_write_group->second,
-								status_line );
+								fmtlib_tools::streamed( status_line ) );
 					} );
 				}
 
@@ -1014,17 +1181,17 @@ class connection_t final
 			{
 				auto wo = m_write_output_ctx.extract_next_write_operation();
 
-				if( holds_alternative< trivial_write_operation_t >( wo ) )
+				if( std::holds_alternative< trivial_write_operation_t >( wo ) )
 				{
-					handle_trivial_write_operation( get< trivial_write_operation_t >( wo ) );
+					handle_trivial_write_operation( std::get< trivial_write_operation_t >( wo ) );
 				}
-				else if( holds_alternative< file_write_operation_t >( wo ) )
+				else if( std::holds_alternative< file_write_operation_t >( wo ) )
 				{
-					handle_file_write_operation( get< file_write_operation_t >( wo ) );
+					handle_file_write_operation( std::get< file_write_operation_t >( wo ) );
 				}
 				else
 				{
-					assert( holds_alternative< none_write_operation_t >( wo ) );
+					assert( std::holds_alternative< none_write_operation_t >( wo ) );
 					finish_handling_current_write_ctx();
 				}
 			}
@@ -1032,7 +1199,8 @@ class connection_t final
 			{
 				trigger_error_and_close( [&]{
 					return fmt::format(
-						"[connection:{}] handle_current_write_ctx failed: {}",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] handle_current_write_ctx failed: {}" ),
 						connection_id(),
 						ex.what() );
 				} );
@@ -1050,10 +1218,11 @@ class connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-							"[connection:{}] sending resp data with "
-							"connection-close attribute "
-							"buf count: {}, "
-							"total size: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] sending resp data with "
+								"connection-close attribute "
+								"buf count: {}, "
+								"total size: {}" ),
 							connection_id(),
 							bufs.size(),
 							op.size() );
@@ -1067,9 +1236,10 @@ class connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-						"[connection:{}] sending resp data, "
-						"buf count: {}, "
-						"total size: {}",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] sending resp data, "
+							"buf count: {}, "
+							"total size: {}" ),
 						connection_id(),
 						bufs.size(),
 						op.size() ); } );
@@ -1090,7 +1260,9 @@ class connection_t final
 							restinio::utils::log_trace_noexcept( m_logger,
 								[&]{
 									return fmt::format(
-											"[connection:{}] outgoing data was sent: {} bytes",
+											RESTINIO_FMT_FORMAT_STRING(
+												"[connection:{}] outgoing data was "
+												"sent: {} bytes" ),
 											connection_id(),
 											written );
 								} );
@@ -1110,9 +1282,10 @@ class connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-							"[connection:{}] sending resp file data with "
-							"connection-close attribute, "
-							"total size: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] sending resp file data with "
+								"connection-close attribute, "
+								"total size: {}" ),
 							connection_id(),
 							op.size() );
 				} );
@@ -1125,7 +1298,8 @@ class connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-						"[connection:{}] sending resp file data, total size: {}",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] sending resp file data, total size: {}" ),
 						connection_id(),
 						op.size() );
 				} );
@@ -1161,7 +1335,9 @@ class connection_t final
 							restinio::utils::log_trace_noexcept( m_logger,
 								[&]{
 									return fmt::format(
-											"[connection:{}] file data was sent: {} bytes",
+											RESTINIO_FMT_FORMAT_STRING(
+												"[connection:{}] file data was sent: "
+												"{} bytes" ),
 											connection_id(),
 											written );
 								} );
@@ -1171,7 +1347,9 @@ class connection_t final
 							restinio::utils::log_error_noexcept( m_logger,
 								[&]{
 									return fmt::format(
-											"[connection:{}] send file data error: {} ({}) bytes",
+											RESTINIO_FMT_FORMAT_STRING(
+												"[connection:{}] send file data error: "
+												"{} ({}) bytes" ),
 											connection_id(),
 											ec.value(),
 											ec.message() );
@@ -1189,7 +1367,8 @@ class connection_t final
 			// Finishing writing this group.
 			m_logger.trace( [&]{
 				return fmt::format(
-						"[connection:{}] finishing current write group",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] finishing current write group" ),
 						this->connection_id() );
 			} );
 
@@ -1200,7 +1379,8 @@ class connection_t final
 			{
 				m_logger.trace( [&]{
 					return fmt::format(
-							"[connection:{}] should keep alive",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] should keep alive" ),
 							this->connection_id() );
 				} );
 
@@ -1222,7 +1402,7 @@ class connection_t final
 					if( m_response_coordinator.empty() )
 					{
 						// Here upgrade req is the only request
-						// to by handled by this connection.
+						// to be handled by this connection.
 						// So it is safe to call a handler for it.
 						handle_upgrade_request();
 					}
@@ -1260,8 +1440,9 @@ class connection_t final
 
 				m_logger.trace( [&]{
 					return fmt::format(
-						"[connection:{}] last sent response was marked "
-						"as complete",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] last sent response was marked "
+							"as complete" ),
 						connection_id() ); } );
 				close();
 			}
@@ -1301,7 +1482,8 @@ class connection_t final
 				{
 					trigger_error_and_close( [&]{
 						return fmt::format(
-							"[connection:{}] unable to write: {}",
+							RESTINIO_FMT_FORMAT_STRING(
+								"[connection:{}] unable to write: {}" ),
 							connection_id(),
 							ec.message() );
 					} );
@@ -1317,7 +1499,8 @@ class connection_t final
 					restinio::utils::log_error_noexcept( m_logger,
 						[&]{
 							return fmt::format(
-								"[connection:{}] notificator error: {}",
+								RESTINIO_FMT_FORMAT_STRING(
+									"[connection:{}] notificator error: {}" ),
 								connection_id(),
 								ex.what() );
 						} );
@@ -1335,7 +1518,7 @@ class connection_t final
 			restinio::utils::log_trace_noexcept( m_logger,
 				[&]{
 					return fmt::format(
-						"[connection:{}] close",
+						RESTINIO_FMT_FORMAT_STRING( "[connection:{}] close" ),
 						connection_id() );
 				} );
 
@@ -1360,7 +1543,8 @@ class connection_t final
 			restinio::utils::log_trace_noexcept( m_logger,
 				[&]{
 					return fmt::format(
-						"[connection:{}] close: close socket",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] close: close socket" ),
 						connection_id() );
 				} );
 
@@ -1370,7 +1554,8 @@ class connection_t final
 			restinio::utils::log_trace_noexcept( m_logger,
 				[&]{
 					return fmt::format(
-						"[connection:{}] close: timer canceled",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] close: timer canceled" ),
 						connection_id() );
 				} );
 
@@ -1379,7 +1564,8 @@ class connection_t final
 			restinio::utils::log_trace_noexcept( m_logger,
 				[&]{
 					return fmt::format(
-						"[connection:{}] close: reset responses data",
+						RESTINIO_FMT_FORMAT_STRING(
+							"[connection:{}] close: reset responses data" ),
 						connection_id() );
 				} );
 
@@ -1463,8 +1649,10 @@ class connection_t final
 					catch( const std::exception & x )
 					{
 						conn_object.trigger_error_and_close( [&] {
-								return fmt::format( "[connection: {}] unexpected "
-										"error during timeout handling: {}",
+								return fmt::format(
+										RESTINIO_FMT_FORMAT_STRING(
+											"[connection: {}] unexpected "
+											"error during timeout handling: {}" ),
 										conn_object.connection_id(),
 										x.what() );
 							} );
@@ -1540,7 +1728,7 @@ class connection_t final
 		{
 			m_logger.trace( [&]{
 				return fmt::format(
-						"[connection:{}] {} timed out",
+						RESTINIO_FMT_FORMAT_STRING( "[connection:{}] {} timed out" ),
 						connection_id(),
 						operation_name );
 			} );
@@ -1622,6 +1810,16 @@ class connection_t final
 
 		//! Logger for operation
 		logger_t & m_logger;
+
+		/*!
+		 * @brief Monitor of the connection lifetime.
+		 *
+		 * It's required for controlling the count of active parallel
+		 * connections.
+		 *
+		 * @since v.0.6.12
+		 */
+		lifetime_monitor_t m_lifetime_monitor;
 };
 
 //
@@ -1635,6 +1833,8 @@ class connection_factory_t
 	public:
 		using logger_t = typename Traits::logger_t;
 		using stream_socket_t = typename Traits::stream_socket_t;
+		using lifetime_monitor_t =
+			typename connection_count_limit_types<Traits>::lifetime_monitor_t;
 
 		connection_factory_t(
 			connection_settings_handle_t< Traits > connection_settings,
@@ -1645,12 +1845,14 @@ class connection_factory_t
 		{}
 
 		// NOTE: since v.0.6.3 it returns non-empty
-		// shared_ptr<connection_t<Traits>> or anexception is thrown in
+		// shared_ptr<connection_t<Traits>> or an exception is thrown in
 		// the case of an error.
+		// NOTE: since v.0.6.12 it accepts yet another parameter: lifetime_monitor.
 		auto
 		create_new_connection(
 			stream_socket_t socket,
-			endpoint_t remote_endpoint )
+			endpoint_t remote_endpoint,
+			lifetime_monitor_t lifetime_monitor )
 		{
 			using connection_type_t = connection_t< Traits >;
 
@@ -1663,7 +1865,8 @@ class connection_factory_t
 				m_connection_id_counter++,
 				std::move( socket ),
 				m_connection_settings,
-				std::move( remote_endpoint ) );
+				std::move( remote_endpoint ),
+				std::move( lifetime_monitor ) );
 		}
 
 	private:
